#include "solaris.h"

#include <kstat.h>

static NetDevInfoVector IfInfo;
static size_t NetDevCount = 0;

NetDevInfo::NetDevInfo()
  : Name(""),
  flags(0),
  ipackets(0),
  OLDipackets(0),
  opackets(0),
  OLDopackets(0),
  ierrors(0),
  OLDierrors(0),
  oerrors(0),
  OLDoerrors(0),
  collisions(0),
  OLDcollisions(0),
  multixmt(0),
  OLDmultixmt(0),
  multircv(0),
  OLDmultircv(0),
  brdcstxmt(0),
  OLDbrdcstxmt(0),
  brdcstrcv(0),
  OLDbrdcstrcv(0)
{
}


NetDevInfo::NetDevInfo(const NetDevInfo& rhs)
  : Name(rhs.Name),
  flags(rhs.flags),
  ipackets(rhs.ipackets),
  OLDipackets(rhs.OLDipackets),
  opackets(rhs.opackets),
  OLDopackets(rhs.OLDopackets),
  ierrors(rhs.ierrors),
  OLDierrors(rhs.OLDierrors),
  oerrors(rhs.oerrors),
  OLDoerrors(rhs.OLDoerrors),
  collisions(rhs.collisions),
  OLDcollisions(rhs.OLDcollisions),
  multixmt(rhs.multixmt),
  OLDmultixmt(rhs.OLDmultixmt),
  multircv(rhs.multircv),
  OLDmultircv(rhs.OLDmultircv),
  brdcstxmt(rhs.brdcstxmt),
  OLDbrdcstxmt(rhs.OLDbrdcstxmt),
  brdcstrcv(rhs.brdcstrcv),
  OLDbrdcstrcv(rhs.OLDbrdcstrcv)
{
}

NetDevInfo::~NetDevInfo()
{
}

NetDevInfo&
NetDevInfo::operator= (const NetDevInfo& rhs)
{
    if (this != &rhs)
    {
	Name = rhs.Name;
	flags = rhs.flags;
	ipackets = rhs.ipackets;
	OLDipackets = rhs.OLDipackets;
	opackets = rhs.opackets;
	OLDopackets = rhs.OLDopackets;
	ierrors = rhs.ierrors;
	OLDierrors = rhs.OLDierrors;
	oerrors = rhs.oerrors;
	OLDoerrors = rhs.OLDoerrors;
	collisions = rhs.collisions;
	OLDcollisions = rhs.OLDcollisions;
	multixmt = rhs.multixmt;
	OLDmultixmt = rhs.OLDmultixmt;
	multircv = rhs.multircv;
	OLDmultircv = rhs.OLDmultircv;
	brdcstxmt = rhs.brdcstxmt;
	OLDbrdcstxmt = rhs.OLDbrdcstxmt;
	brdcstrcv = rhs.brdcstrcv;
	OLDbrdcstrcv = rhs.OLDbrdcstrcv;
    }
    return *this;
}

int getnetdevcount()
{
    return NetDevCount;
}

const NetDevInfoVector& getnetdevvector()
{
    return IfInfo;
}

int insertnetdev (const char *name, short flags)
{
        int     i = 0;

        /*
         *  interface "aliases" don't seem to have
         *  separate kstat statistics, so we skip them
         */
        if (::strchr(name, ':' ) != NULL)
                return(0);

        while ((i < NetDevCount) && (::strcmp(IfInfo[i].Name.c_str(), name) != 0))
	{
                if (::strcmp( IfInfo[i].Name.c_str(), name) == 0)
                        return(0);
                i++;
        }
	NetDevInfo netInfo;

        netInfo.Name = name;
        netInfo.flags = flags;
        netInfo.ipackets = 0L;
        netInfo.OLDipackets = 0L;
        netInfo.opackets = 0L;
        netInfo.OLDopackets = 0L;
        netInfo.ierrors = 0L;
        netInfo.OLDierrors = 0L;
        netInfo.oerrors = 0L;
        netInfo.OLDoerrors = 0L;
        netInfo.collisions = 0L;
        netInfo.OLDcollisions = 0L;
        netInfo.multixmt = 0L;
        netInfo.OLDmultixmt = 0L;
        netInfo.multircv = 0L;
        netInfo.OLDmultircv = 0L; 
        netInfo.brdcstxmt = 0L;
        netInfo.OLDbrdcstxmt = 0L;
        netInfo.brdcstrcv = 0L;
        netInfo.OLDbrdcstrcv = 0L;
	IfInfo.push_back(netInfo);
        NetDevCount = static_cast<int>(IfInfo.size());
	return 0;
}

int getnetdevlist()
{
        int             fd;
        int             buffsize;
        int             prevsize;
        int             prevCount;
        struct ifconf   ifc;
        struct ifreq    *ifr;

        if ( (fd = socket( PF_INET, SOCK_DGRAM, 0 )) < 0 )
	{
                return( -1 );
        }

        /*
         *  get the interface list via iotl( SIOCGIFCONF )
         *  the following algorithm based on ideas from W.R. Stevens'
         *  "UNIX Network Programming", Vol. 1:
         *  Since the ioctl may return 0, indicating success, even if the
         *  ifreq buffer was too small, we have to make sure, it didn't
         *  get truncated by comparing our initial size guess with the
         *  actual returned size.
         */
        prevsize = 0;
        buffsize = NBUFFERS * sizeof( struct ifreq );
        while( 1 )
	{
                if( (ifc.ifc_buf = (char *) ::malloc( buffsize )) == NULL )
                        return( -1 );

                ifc.ifc_len = buffsize;
                if( ioctl( fd, SIOCGIFCONF, &ifc ) < 0 )
		{
                        if( errno != EINVAL || prevsize != 0 )
			{
                                free( ifc.ifc_buf );
                                return( -1 );
                        }
                }
		else
		{
                        if( ifc.ifc_len == prevsize )
                                /*  success  */
                                break;
                        prevsize = ifc.ifc_len;
                }
                /*
                 *  initial buffer guessed too small, allocate a bigger one
                 */
                free( ifc.ifc_buf );
                buffsize = (NBUFFERS + 10) * sizeof( struct ifreq );
        }

        /*
         *  get the names for all interfaces which are configured "up"
         *  we're not interested in the ifc data (address), so we reuse the
         *  same structure (with ifc.len set) for the next ioctl()
         */
        prevCount = NetDevCount;
        for( ifr = (struct ifreq *) ifc.ifc_buf;
                        ifr < (struct ifreq *) (ifc.ifc_buf + ifc.ifc_len);
                        ifr++ ) {
                if( ioctl( fd, SIOCGIFFLAGS, ifr ) < 0 ) {
                        free( ifc.ifc_buf );
                        return( -1 );
                }
                if( ifr->ifr_flags & IFF_UP )
                        insertnetdev( ifr->ifr_name, ifr->ifr_flags );
        }
        free( ifc.ifc_buf );
        close( fd );
        if( (prevCount > 0) && (prevCount != NetDevCount) ) {
                prevCount = NetDevCount;
        }
        return( NetDevCount );
}

int updatenetdev()
{
        kstat_ctl_t             *kctl;
        kstat_t                 *ksp;
        kstat_named_t           *kdata;
        int                     i;

        /*
         *  get a kstat handle and update the user's kstat chain
         */
        if( (kctl = kstat_open()) == NULL )
                return( 0 );
        while( kstat_chain_update( kctl ) != 0 )
                ;

        for( i = 0; i < NetDevCount; i++ )
	{
                char    *name;
                char    *ptr;

                /*
                 *  chop off the trailing interface no
                 */
                name = ::strdup(IfInfo[i].Name.c_str());
                ptr = name + strlen( name ) - 1;
                while( (ptr > name) && isdigit( (int) *ptr ) )
		{
                        *ptr = '\0';
                        ptr--;
                }

                /*
                 *  traverse the kstat chain
                 *  to find the appropriate statistics
                 */
                if( (ksp = kstat_lookup( kctl,
                                name, 0, (char *) IfInfo[i].Name.c_str() )) == NULL ) {
                        free( name );
                        return( 0 );
                }
                if( kstat_read( kctl, ksp, NULL ) == -1 ) {
                        free( name );
                        return( 0 );
                }
                free(name);

                /*
                 *  lookup & store the data
                 */
                kdata = (kstat_named_t *) kstat_data_lookup( ksp, "ipackets" );
                if( kdata != NULL ) {
                        IfInfo[i].OLDipackets = IfInfo[i].ipackets;
                        IfInfo[i].ipackets = kdata->value.ul;
                }
                kdata = (kstat_named_t *) kstat_data_lookup( ksp, "opackets" );
                if( kdata != NULL ) {
                        IfInfo[i].OLDopackets = IfInfo[i].opackets;
                        IfInfo[i].opackets = kdata->value.ul;
                }
                kdata = (kstat_named_t *) kstat_data_lookup( ksp, "ierrors" );
                if( kdata != NULL ) {
                        IfInfo[i].OLDierrors = IfInfo[i].ierrors;
                        IfInfo[i].ierrors = kdata->value.ul;
                }
                kdata = (kstat_named_t *) kstat_data_lookup( ksp, "oerrors" );
                if( kdata != NULL ) {
                        IfInfo[i].OLDoerrors = IfInfo[i].oerrors;
                        IfInfo[i].oerrors = kdata->value.ul;
                }
                kdata = (kstat_named_t *) kstat_data_lookup( ksp, "collisions" );
                if( kdata != NULL ) {
                        IfInfo[i].OLDcollisions = IfInfo[i].collisions;
                        IfInfo[i].collisions = kdata->value.ul;
                }
                kdata = (kstat_named_t *) kstat_data_lookup( ksp, "multixmt" );
                if( kdata != NULL ) {
                        IfInfo[i].OLDmultixmt = IfInfo[i].multixmt;
                        IfInfo[i].multixmt = kdata->value.ul;
                }
                kdata = (kstat_named_t *) kstat_data_lookup( ksp, "multircv" );
                if( kdata != NULL ) {
                        IfInfo[i].OLDmultircv = IfInfo[i].multircv;
                        IfInfo[i].multircv = kdata->value.ul;
                }
                kdata = (kstat_named_t *) kstat_data_lookup( ksp, "brdcstxmt" );
                if( kdata != NULL ) {
                        IfInfo[i].OLDbrdcstxmt = IfInfo[i].brdcstxmt;
                        IfInfo[i].brdcstxmt = kdata->value.ul;
                }
                kdata = (kstat_named_t *) kstat_data_lookup( ksp, "brdcstrcv" );
                if( kdata != NULL ) {
                        IfInfo[i].OLDbrdcstrcv = IfInfo[i].brdcstrcv;
                        IfInfo[i].brdcstrcv = kdata->value.ul;
                }
        }
        kstat_close( kctl );
        return( 0 );
}


